# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/FEWS/Modules/Waterlevels/waterlevels/ReadMetaDataAdjusted.py $
# Last committed:     $Revision: 394467 $; $Author: ahove $; $Date: 2019-10-29 10:04:37 +0100 (di, 29 okt 2019) $
#
# -*- coding: utf-8 -*-

# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/FEWS/Modules/Waterlevels/waterlevels/ReadMetaDataAdjusted.py $
# Last committed:     $Revision: 394467 $; $Author: ahove $; $Date: 2019-10-29 10:04:37 +0100 (di, 29 okt 2019) $

"""
Andere versies van dit script staan op locaties:
    C:\dms\code\python-ftp-svn\source\mics\ReadMetaData.py
    C:\dms\code\nc-to-thredds\source\ReadMetaDataAdjusted_NoLog.py
"""

import numpy as np
import pandas as pd
import re


class ReadMetaDataAdjusted:

    def __init__(self, fn, ts, log, request_location):
        self.fn = fn
        self.ts = ts
        self.log = log
        self.current_meta_data = {}
        self.parse_file(request_location)

    def insert_tuple(self, k, t, v):
        if k not in self.current_meta_data:
            self.current_meta_data[k] = {t: v}
        else:
            tmpTup = self.current_meta_data[k]
            tmpTup[t] = v
            self.current_meta_data[k] = tmpTup

    def parse_file(self, request_location):
        self.log.debug("ReadMetaDataAdjusted.parse_file: reading".format(self.fn))
        xls = pd.ExcelFile(self.fn)

        self.log.debug("ReadMetaDataAdjusted.parse_file: looping through sheets")
        for sheet in xls.sheet_names:
            self.log.debug("ReadMetaDataAdjusted.parse_file: reading sheet {}".format(sheet))
            df = pd.read_excel(self.fn, sheet_name=sheet)
            df['NAP Z'] = df['NAP Z'].map(lambda x: np.nan if isinstance(x, str) else x)
            # print("{}: {}".format(sheet, df.columns))

            # get location from sheet name
            if sheet == 'BUOY1':
                m = re.match('^(BUOY\d{1})(\w?)', sheet)
            else:
                m = re.match('^(FL\d{2})(\w?)', sheet)
            if m is None:
                self.log.error("ReadMetaDataAdjusted.parse_file: could not retrieve location name from sheet name {}".format(sheet))
                raise
            locatie = m.group(1)
            # location FMW is known as BUOY1 in FEWS and on the file system
            ##if locatie == 'FMW':
            #    self.log.debug("ReadMetaDataAdjusted.parse_file: location {} modified to BUOY1".format(locatie))
            #    location = 'BUOY1'
            # sub-locations
            if m.group(2) != "":
                locatie = locatie + '_' + m.group(2)

            # skip in case the requested location is a different one
            if request_location != locatie:
                continue

            # dataframe of the input parameter ts (timestamps)
            df_datum = pd.DataFrame(data={'Datum': self.ts})

            df_bordes1 = df[df.Item.str.match("^Bordes$", flags=re.IGNORECASE)]
            if not df_bordes1.empty:
                df_bordes = pd.merge_asof(left=df_datum, right=df_bordes1[['Datum', 'RD X', 'RD Y']], on='Datum', direction='backward')
                df_bordes.fillna(df_bordes1.iloc[0][['RD X', 'RD Y']], inplace=True)
                x = df_bordes['RD X'].values
                y = df_bordes['RD Y'].values
                name = df_bordes1.iloc[0]['Locatie']
                # print("df_bordes {}: {}, {}".format(locatie, x, y))
                self.insert_tuple(locatie, 'X', x)
                self.insert_tuple(locatie, 'Y', y)
                self.insert_tuple(locatie, 'Naam', name)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('X', x))
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('Y', y))
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('Naam', name))

            df_bodem1 = df[df.Item.str.match("^bodem", flags=re.IGNORECASE)]
            if not df_bodem1.empty:
                df_bodem = pd.merge_asof(left=df_datum, right=df_bodem1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_bodem.fillna(df_bodem1.iloc[0][['NAP Z']], inplace=True)
                nap_z = df_bodem['NAP Z'].values
                self.insert_tuple(locatie, 'bodem_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('bodem_NAP_Z', nap_z))

            df_alt1 = df[(df.Item.str.match("^ALT", flags=re.IGNORECASE))]
            df_alt1[df_alt1['NAP Z'].notnull()]
            if not df_alt1.empty:
                df_alt = pd.merge_asof(left=df_datum, right=df_alt1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_alt.fillna(df_alt1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_alt['NAP Z'].values
                # print("df_alt {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ALT_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ALT_NAP_Z', nap_z))

            df_stb1 = df[(df.Item.str.match("^STB", flags=re.IGNORECASE))]
            df_stb1[df_stb1['NAP Z'].notnull()]
            if not df_stb1.empty:
                df_stb = pd.merge_asof(left=df_datum, right=df_stb1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_stb.fillna(df_stb1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_stb['NAP Z'].values
                # print("df_stb {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'STB_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('STB_NAP_Z', nap_z))

            df_adc1 = df[(df.Item.str.match("^ADC", flags=re.IGNORECASE))]
            df_adc1[df_adc1['NAP Z'].notnull()]
            if not df_adc1.empty:
                df_adc = pd.merge_asof(left=df_datum, right=df_adc1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_adc.fillna(df_adc1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adc['NAP Z'].values
                # print("df_adc {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ADC_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADC_NAP_Z', nap_z))

            df_adv1 = df[(df.Item.str.match("^ADV", flags=re.IGNORECASE))]
            df_adv1[df_adv1['NAP Z'].notnull()]
            if not df_adv1.empty:
                df_adv = pd.merge_asof(left=df_datum, right=df_adv1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_adv.fillna(df_adv1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adv['NAP Z'].values
                # print("df_adv {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ADV_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('ADV_NAP_Z', nap_z))

            df_mpp1 = df[(df.Item.str.match("^MPP", flags=re.IGNORECASE))]
            df_mpp1[df_mpp1['NAP Z'].notnull()]
            if not df_mpp1.empty:
                df_mpp = pd.merge_asof(left=df_datum, right=df_mpp1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_mpp.fillna(df_mpp1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_mpp['NAP Z'].values
                # print("df_mpp {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'MPP_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('MPP_NAP_Z', nap_z))

            df_aqd1 = df[(df.Item.str.match("^AQD", flags=re.IGNORECASE))]
            df_aqd1[df_aqd1['NAP Z'].notnull()]
            if not df_aqd1.empty:
                df_aqd = pd.merge_asof(left=df_datum, right=df_aqd1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_aqd.fillna(df_aqd1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_aqd['NAP Z'].values
                # print("df_aqd {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'AQD_NAP_Z', nap_z)
                self.log.debug("ReadMetaDataAdjusted.parse_file: variables parsed {}: {}".format('AQD_NAP_Z', nap_z))

    def get_meta_data(self):
        return self.current_meta_data
